'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.canUserSubscribePublication = exports.canUserExecuteMethod = exports.addComponentToRole = exports.addPublicationToRole = exports.addMethodToRole = exports.getRole = exports.editRole = exports.removeRole = exports.addRole = exports.removeUserFromRole = exports.addUserToRole = exports.UserInRole = exports.Role = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; // import { Mongo } from 'meteor/mongo';
// import { check, Match } from 'meteor/check';


var _ramda = require('ramda');

var _random = require('./random.js');

var _method = require('./method.js');

var _publication = require('./publication.js');

var Role = exports.Role = new Mongo.Collection('meteor_abac_roles');
var UserInRole = exports.UserInRole = new Mongo.Collection('meteor_abac_userinrole');

var isString = function isString(val) {
  return Object.prototype.toString.call(val) === '[object String]';
};

var doesRoleExist = function doesroleexist(role) {
  return !!Role.findOne({
    name: role
  });
};

var isUserInRole = function isUserInRole(userId, roles) {
  check(userId, Match.OneOf(String, null));

  check(roles, Match.OneOf(String, [String]));

  var rs = roles;

  if (isString(roles)) {
    rs = [roles];
  }

  return UserInRole.find({
    userId: userId,
    role: {
      $in: rs
    }
  }).count() > 0;
};

var findRolesByMethod = function findRolesByMethod(methodName) {
  return Role.find({
    method: methodName
  }).fetch();
};

var findRolesByPublication = function findRolesByPublication(pubName) {
  return Role.find({
    publication: pubName
  }).fetch();
};

var addUserToRole = exports.addUserToRole = function addUserToRole(userIds, roles) {
  check(userIds, Match.OneOf(String, [String]));

  check(roles, Match.OneOf(String, [String]));

  var uids = userIds;
  var rs = roles;

  if (isString(userIds)) {
    uids = [userIds];
  }

  if (isString(roles)) {
    rs = [roles];
  }

  rs.forEach(function (r) {
    if (!doesRoleExist(r)) {
      throw new Meteor.Error('not-existed', 'Role ' + r + ' does not exist');
    }
  });

  var userInRoles = uids.map(function (userId) {
    return rs.map(function (role) {
      return {
        userId: userId,
        role: role
      };
    });
  });

  var filterList = (0, _ramda.flatten)(userInRoles).filter(function (uir) {
    return !UserInRole.findOne(uir);
  });

  if (filterList.length > 0) {
    (function () {
      var bulk = UserInRole.rawCollection().initializeUnorderedBulkOp();
      var now = new Date();

      filterList.forEach(function (uir) {
        bulk.insert(_extends({
          _id: (0, _random.id)()
        }, uir, {
          createdAt: now,
          modifiedAt: now
        }));
      });

      bulk.execute(function () {});
    })();
  }
};

var removeUserFromRole = exports.removeUserFromRole = function removeUserFromRole(userIds, roles) {
  check(userIds, Match.OneOf(String, [String]));

  check(roles, Match.OneOf(String, [String]));

  var uids = userIds;
  var rs = roles;

  if (isString(userIds)) {
    uids = [userIds];
  }

  if (isString(roles)) {
    rs = [roles];
  }

  UserInRole.remove({
    userId: {
      $in: uids
    },
    role: {
      $in: rs
    }
  });
};

var addRole = exports.addRole = function addRoles(role) {
  check(role, {
    name: String,
    method: Match.Optional([String]),
    publication: Match.Optional([String]),
    component: Match.Optional([String])
  });

  var now = new Date();

  Role.upsert({
    name: role.name
  }, {
    $set: _extends({
      method: [],
      publication: [],
      component: []
    }, role, {
      createdAt: now,
      modifiedAt: now
    })
  });
};

var removeRole = exports.removeRole = function removeRole(role) {
  check(role, Match.OneOf(String, [String]));

  var r = role;

  if (isString(role)) {
    r = [role];
  }

  Role.remove({
    name: {
      $in: r
    }
  });

  UserInRole.remove({
    role: {
      $in: r
    }
  });
};

var editRole = exports.editRole = function editRole(role) {
  check(role, {
    _id: String,
    name: Match.Optional(String),
    method: Match.Optional([String]),
    publication: Match.Optional([String]),
    component: Match.Optional([String])
  });

  var now = new Date();

  Role.update({
    _id: role._id
  }, {
    $set: _extends({}, role, {
      modifiedAt: now
    })
  });
};

var getRole = exports.getRole = function getRole(idOrName) {
  check(idOrName, String);

  return Role.findOne({
    $or: [{
      _id: idOrName
    }, {
      name: idOrName
    }]
  });
};

var addMethodToRole = exports.addMethodToRole = function addMethodToRole(roleNameOrId, methods) {
  check(roleNameOrId, String);
  check(methods, Match.OneOf(String, [String]));

  var m = methods;

  if (isString(methods)) {
    m = [methods];
  }

  if (!doesRoleExist(roleNameOrId)) {
    throw new Meteor.Error('not-existed', 'Role ' + roleNameOrId + ' does not eixst');
  }

  m.forEach(function (obj) {
    if (!(0, _method.doesMethodExist)(obj)) {
      throw new Meteor.Error('not-existed', 'Method ' + m + ' does not exist');
    }
  });

  Role.update({
    $or: [{
      _id: roleNameOrId
    }, {
      name: roleNameOrId
    }]
  }, {
    $push: {
      method: {
        $each: m
      }
    }
  });
};

var addPublicationToRole = exports.addPublicationToRole = function addPublicationToRole(roleNameOrId, pub) {
  check(roleNameOrId, String);
  check(pub, Match.OneOf(String, [String]));

  var m = pub;

  if (isString(pub)) {
    m = [pub];
  }

  if (!doesRoleExist(roleNameOrId)) {
    throw new Meteor.Error('not-existed', 'Role ' + roleNameOrId + ' does not eixst');
  }

  m.forEach(function (obj) {
    if (!(0, _publication.doesPublicationExist)(obj)) {
      throw new Meteor.Error('not-existed', 'Publication ' + m + ' does not exist');
    }
  });

  Role.update({
    $or: [{
      _id: roleNameOrId
    }, {
      name: roleNameOrId
    }]
  }, {
    $push: {
      publication: {
        $each: m
      }
    }
  });
};

var addComponentToRole = exports.addComponentToRole = function addComponentToRole(roleNameOrId, component) {
  check(roleNameOrId, String);
  check(component, Match.OneOf(String, [String]));

  var m = component;

  if (isString(component)) {
    m = [component];
  }

  if (!doesRoleExist(roleNameOrId)) {
    throw new Meteor.Error('not-existed', 'Role ' + roleNameOrId + ' does not eixst');
  }

  Role.update({
    $or: [{
      _id: roleNameOrId
    }, {
      name: roleNameOrId
    }]
  }, {
    $push: {
      component: {
        $each: m
      }
    }
  });
};

var canUserExecuteMethod = exports.canUserExecuteMethod = function canUserExecuteMethod(userId, method) {
  var rolesCanDoOp = findRolesByMethod(method).map(function (obj) {
    return obj.name;
  });

  return isUserInRole(userId, rolesCanDoOp);
};

var canUserSubscribePublication = exports.canUserSubscribePublication = function canUserSubscribePublication(userId, pub) {
  var rolesCanSubscribe = findRolesByPublication(pub).map(function (obj) {
    return obj.name;
  });

  return isUserInRole(userId, rolesCanSubscribe);
};